from numpy import polysub, zeros, polymul, polyadd
import numpy as np
from scipy.signal import lfilter


def FilterAdd(f1, f2):
    den = polymul(f1.get_den(), f2.get_den())
    n1 = polymul(f1.get_num(), f2.get_den())
    n2 = polymul(f1.get_den(), f2.get_num())
    num = polyadd(n1, n2)
    return Filter(num, den)


def FilterSub(f1, f2):
    den = polymul(f1.get_den(), f2.get_den())
    n1 = polymul(f1.get_num(), f2.get_den())
    n2 = polymul(f1.get_den(), f2.get_num())
    num = polysub(n1, n2)
    return Filter(num, den)


def FilterMult(f1, f2):
    return Filter(polymul(f1.get_num(), f2.get_num()), polymul(f1.get_den(), f2.get_den()))


class Filter:
    def __init__(self, b, a):
        self.b = b
        self.a = a

    def get_num(self):
        return self.b

    def get_den(self):
        return self.a

    def apply(self, x):
        return lfilter(self.b, self.a, x)


class AllPass:
    def __init__(self, b, a):
        self.b = b
        self.a = a

    def apply(self, x):
        return lfilter(self.b, self.a, x)

    def order(self):
        return len(self.a) - 1

    def get_transfer_function(self):
        return Filter(self.b, self.a)


class AllPassChain:
    def __init__(self, filter_list=[]):
        self.filters = filter_list

    def order(self):
        return self.filters[0].order()

    def append(self, filter):
        self.filters.append(filter)

    def apply_chain(self, x):
        for i in self.filters:
            x = i.apply(x)

        return x

    def __str__(self):
        for i in self.filters:
            return "b: " + str(i.b) + " a: " + str(i.a)

    def get_transfer_function(self):
        den = 1
        num = 1
        for i in self.filters:
            den = polymul(den, i.a)
            num = polymul(num, i.b)
        return Filter(num, den)


class Delay:
    def __init__(self, order):
        self.order = order

    def get_transfer_function(self):
        b = np.zeros(self.order + 1)
        a = np.zeros(self.order + 1)
        b[len(b) - 1] = 1
        a[0] = 1
        return Filter(b, a)

    def get_den(self):
        return self.get_transfer_function().get_den()

    def get_num(self):
        return self.get_transfer_function().get_num()


class LowPass:
    def __init__(self, coef):
        bi = []
        by = []
        for i in coef[0::2]:
            bi.append(AllPass([i, 0, 1], [1, 0, i]))
        for i in coef[1::2]:
            by.append(AllPass([i, 0, 1], [1, 0, i]))
        self.bi = AllPassChain(bi)
        self.by = AllPassChain(by)

    def get_transfer_function(self):
        bi_tf = self.bi.get_transfer_function()
        by_tf = self.by.get_transfer_function()
        dl = Delay(1).get_transfer_function()

        by_tf = FilterMult(dl, by_tf)

        out = FilterAdd(bi_tf, by_tf)
        out = Filter(out.get_num(), 2 * out.get_den())

        return out

    def get_num(self):
        return self.get_transfer_function().get_num()

    def get_den(self):
        return self.get_transfer_function().get_den()


class HighPass:
    def __init__(self, coef):
        bi = []
        by = []
        for i in coef[0::2]:
            bi.append(AllPass([i, 0, 1], [1, 0, i]))
        for i in coef[1::2]:
            by.append(AllPass([i, 0, 1], [1, 0, i]))
        self.bi = AllPassChain(bi)
        self.by = AllPassChain(by)

    def get_transfer_function(self):
        bi_tf = self.bi.get_transfer_function()
        by_tf = self.by.get_transfer_function()
        dl = Delay(1).get_transfer_function()

        by_tf = FilterMult(dl, by_tf)

        out = FilterSub(bi_tf, by_tf)
        out = Filter(out.get_num(), 2 * out.get_den())

        return out

    def get_num(self):
        return self.get_transfer_function().get_num()

    def get_den(self):
        return self.get_transfer_function().get_den()


class Hilbert:
    def __init__(self, coef):
        bi = []
        by = []
        for i in coef[0::2]:
            bi.append(AllPass([i, 0, -1], [1, 0, -i]))
        for i in coef[1::2]:
            by.append(AllPass([i, 0, -1], [1, 0, -i]))
        self.bi = AllPassChain(bi)
        self.by = AllPassChain(by)

    def get_transfer_function(self):
        bx_tf = self.bi.get_transfer_function()
        by_tf = self.by.get_transfer_function()
        dl = Delay(1).get_transfer_function()
        by_tf = FilterMult(dl, by_tf)
        f = Filter([1], [np.sqrt(2)])  # 0.5
        bx = FilterMult(f, FilterAdd(by_tf, bx_tf))
        f = Filter([1], [np.sqrt(2)])  # 0.5
        by = FilterMult(f, FilterSub(by_tf, bx_tf))
        return bx, by

    def apply(self, x):
        hi, hq = self.get_transfer_function()
        hx = lfilter(hi.b, hi.a, x)
        hy = lfilter(hq.b, hq.a, x)
        l = []
        for i, j in zip(hx, hy):
            l.append(i + 1j*j)
        return np.array(l)



#!/usr/bin/env python3

import ctypes as ct
import numpy as np
import os

if os.name == 'nt':
    path = 'hiir.dll'
else:
    path = 'libhiir.so'

class hiir:
    def __init__(self):
        libname = os.path.dirname(os.path.abspath(__file__)) + os.path.sep
        libname += path.replace('/', os.path.sep)

        self.lib = ct.cdll.LoadLibrary(libname)

        self.lib.iir2designer_compute_nbr_coefs_from_proto.restype = ct.c_int
        self.lib.iir2designer_compute_nbr_coefs_from_proto.argtypes = [ct.c_double, ct.c_double]

        self.lib.iir2designer_compute_coefs.restype = ct.c_int
        self.lib.iir2designer_compute_coefs.argtypes = [ct.POINTER(ct.c_double), ct.c_double, ct.c_double]

        self.lib.iir2designer_compute_coefs_spec_order_tbw.argtypes = [ct.POINTER(ct.c_double), ct.c_int, ct.c_double]


    def compute_nbr_coefs_from_proto(self, attenuation, transition):
        return self.lib.iir2designer_compute_nbr_coefs_from_proto(ct.c_double(attenuation), ct.c_double(transition))

    def compute_coefs(self, attenuation, transition):
        n = self.compute_nbr_coefs_from_proto(attenuation, transition)
        x = np.zeros(n, dtype=np.double)
        self.lib.iir2designer_compute_coefs(x.ctypes.data_as(ct.POINTER(ct.c_double)).contents, ct.c_double(attenuation), ct.c_double(transition))
        return x

    def compute_coefs_order_tbw(self, n, transition):
        x = np.zeros(n, dtype=np.double)
        self.lib.iir2designer_compute_coefs_spec_order_tbw(x.ctypes.data_as(ct.POINTER(ct.c_double)).contents, ct.c_int(n), ct.c_double(transition))
        return x

if __name__=='__main__':
    pass


import os
import re
import sys
import sysconfig
import platform
import subprocess

from distutils.version import LooseVersion
from setuptools import setup, find_packages, Extension
from setuptools.command.build_ext import build_ext


class CMakeExtension(Extension):
    def __init__(self, name, sourcedir=''):
        Extension.__init__(self, name, sources=[])
        self.sourcedir = os.path.abspath(sourcedir)


class CMakeBuild(build_ext):
    def run(self):
        try:
            out = subprocess.check_output(['cmake', '--version'])
        except OSError:
            raise RuntimeError(
                "CMake must be installed to build the following extensions: " +
                ", ".join(e.name for e in self.extensions))

        if platform.system() == "Windows":
            cmake_version = LooseVersion(re.search(r'version\s*([\d.]+)',
                                         out.decode()).group(1))
            if cmake_version < '3.20.0':
                raise RuntimeError("CMake >= 3.20.0 is required on Windows")

        for ext in self.extensions:
            self.build_extension(ext)

    def build_extension(self, ext):
        extdir = os.path.abspath(
            os.path.dirname(self.get_ext_fullpath(ext.name)))
        cmake_args = ['-DCMAKE_LIBRARY_OUTPUT_DIRECTORY=' + extdir, ]#'-DPYTHON_EXECUTABLE=' + sys.executable]

        cfg = 'Debug' if self.debug else 'Release'
        build_args = ['--config', cfg]

        if platform.system() == "Windows":
            cmake_args += ['-DCMAKE_LIBRARY_OUTPUT_DIRECTORY_{}={}'.format(
                cfg.upper(),
                extdir)]
        else:
            cmake_args += ['-DCMAKE_BUILD_TYPE=' + cfg]

        env = os.environ.copy()
        env['CXXFLAGS'] = '{} -DVERSION_INFO=\\"{}\\"'.format(
            env.get('CXXFLAGS', ''),
            self.distribution.get_version())
        if not os.path.exists(self.build_temp):
            os.makedirs(self.build_temp)
        subprocess.check_call(['cmake', ext.sourcedir] + cmake_args,
                              cwd=self.build_temp, env=env)
        subprocess.check_call(['cmake', '--build', '.'] + build_args,
                              cwd=self.build_temp)
        print()  # Add an empty line for cleaner output

class git_clone_external(build_ext):
    def run(self):
        subprocess.check_call(['git', 'clone', 'https://git.example.com'])
        build_ext.run(self)

with open('README.md', 'r', encoding='utf-8') as f:
    long_description = f.read()

def get_git_version():
    try:
        version = subprocess.check_output(['git', 'describe', '--tags', '--always'])
        version = version.decode('utf-8').strip().strip('v')
    except:
        raise RuntimeError("Unable to get version number from git tags")
    return version

setup(
    name='pyhiir',
    version=get_git_version(),
    author='Diego Asanza',
    author_email='diego.asanza@gmail.com',
    description='Python wrapper for HIIR library',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/asanza/pyhiir',
    # tell setuptools to look for any packages under 'src'
    packages=find_packages('src'),
    # tell setuptools that all packages will be under the 'src' directory
    # and nowhere else
    package_dir={'':'src'},
    # add an extension module named 'python_cpp_example' to the package 
    # 'python_cpp_example'
    ext_modules=[CMakeExtension('pyhiir/pyhiir')],
    # add custom build_ext command
    cmdclass=dict(build_ext=CMakeBuild),
    install_requires = [
        'numpy >= 1.19.2',
        'scipy >= 1.6.3',
    ],
    zip_safe=False,
    test_suite='tests',
)